@@ -1,67 +1,68 @@
 package Carp::Notify;
 
-#Copyright (c) 2000 James A Thomason III (thomasoniii@yahoo.com). All rights reserved.
+#Copyright (c) 2000, 2001 James A Thomason III (thomasoniii@yahoo.com). All rights reserved.
 #This program is free software; you can redistribute it and/or
 #modify it under the same terms as Perl itself.
 
+$VERSION = "1.10";
 
-$VERSION = "1.00";
+use 5.005;	#probably lower, but I haven't tested it below 005
+use strict;
+local $^W = 1;
 
-use 5.004;	#probably lower, but I haven't tested it below 004
-#use Socket;
-#use strict;
-#$^W = 1;
+my %def = (
+	"smtp"			=> 'your.smtp.com',			#IMPORTANT!  Set this!  I mean it!
+	"domain"		=> 'smtp.com',				#IMPORTANT!  Set this!  I mean it!
+	"port"			=> 25,
+	
+	"email_it"		=> 1,						#should we email by default?	true/false
+	"email"			=> 'someone@your.smtp.com',	#who are we emailing by default?
+	"return"		=> 'someone@your.smtp.com',	#who is the error coming from?
+	"subject"		=> 'Ye Gods!  An error!',
+	
+	"log_it"		=> 1,						#should we log by default?
+	"log_file"		=> '/tmp/error.log',		#default error log for notifys and explodes
 
-my $def_smtp 		= "your.smtp.com";	#<--IMPORTANT!  Set this!
+	"log_explode"	=> 0,						#should we log explodes by default?
+	"explode_log"	=> '/tmp/explode.log',		#default error log for explodes ONLY
 
-my $def_email_it	= 1;
-my $def_email 		= 'thomasoniii@yahoo.com';
-my $def_return		= 'thomasoniii@yahoo.com';
-my $def_subject 	= "Ye Gods!  An error!";
-my $def_domain		= "smtp.com";		#<--IMPORTANT!  Set this! I mean it!
+	"log_notify"	=> 0,						#should we log notifys by default?
+	"notify_log"	=> '/tmp/notify.log',		#default error log for notifys ONLY
+	
+	"store_vars"	=> 1,						#should we store variables by default? true/false
+	"stack_trace"	=> 1,						#should we do a stack trace by default? true/false
+	"store_env"		=> 1,						#should we store our environment by default? true/false
+	
+	"die_to_stdout"	=> 0,						#should we send our death_message to STDOUT by default? true/false
+	"die_everywhere"=> 0,						#should we send our death_message to STDOUT and STDERR by default? true/false
+	"die_quietly"	=> 0,						#should we not print our death_message anywhere?  true/false
+	
+	"error_function"=> '',#'&main::error',		#function to call if Carp::Notify encounters an error
+	"death_function"=> '',#'&main::death',		#function to call upon termination, used in place of death_message
+	
+	#What would you like to die with?  This is probably the message that's going to your user in
+	#his browser, so make it something nice.  You'll have to set the content type yourself, though.
+	#Why's that, you ask?  I wanted to be sure that you had the option of easily redirecting to 
+	#a different page if you'd prefer.
 
-my $def_log_it 		= 1;
-my $def_log_file 	= "error.log";
-
-my $def_store_vars	= 1;
-my $def_stack_trace	= 1;
-my $def_store_env	= 1;
-
-#die_to_stdout is important.  Since a lot of CGI scripts will be using this thing, it will be
-#very helpful to die with some message to the browser, if so desired.  Of course, this module
-#is also very useful for cron jobs or other things that are running in the background where
-#we probably don't want to die to STDOUT, or possibly at all.  Set everything here to reflect
-#that
-
-my $def_die_to_stdout	= 0;
-
-#Don't die with a message to STDERR or STDOUT.  Just let the explosion log and the email
-#suffice.
-my $def_die_quietly		= 0;
-
-#What would you like to die with?  This is probably the message that's going to your user in
-#his browser, so make it something nice.  You'll have to set the content type yourself, though.
-#Why's that, you ask?  I wanted to be sure that you had the option of easily redirecting to 
-#a different page if you'd prefer.
-
-my $def_death_message   = <<eoE;
+	"death_message"   => <<'eoE'
 Content-type:text/plain\n\n
 
 We're terribly sorry, but some horrid internal error seems to have occurred.  We are actively
 looking into the problem and hope to repair the error shortly.  We're sorry for any inconvenience.
 
 eoE
+);
+#end defaults.  Don't mess with anything else! I mean it!
 
-#end defaults.  Don't mess with anything else!
-
-my $settables = "(?:log_it|email_it|store_vars|stack_trace|store_env|email|log_file|smtp|die_to_stdout|die_quietly|death_message)";
+my $settables = "(?:" . join('|', keys %def) . ')';
 
 BEGIN {
 	$Carp::Notify::can_email = 1;
 	eval "use Socket";
 	$Carp::Notify::can_email = 0 if $@;
 	
-	$Carp::Notify::fatal = 1;	#doesn't really belong here, but the cleanest place to put it...
+	$Carp::Notify::fatal = 1;	#doesn't really belong here, but it's the cleanest place to put it...
 };
 
 
@@ -74,7 +75,6 @@ BEGIN {
 	my @storable_vars = ();
 	my %init = ();
 	
-	
 	sub import {
 		my ($package, $file, $line) = caller;
 		$calling_package = $package;
@@ -82,27 +82,26 @@ BEGIN {
 		*{$package . "::explode"} = \&Carp::Notify::explode;
 		*{$package . "::notify"} = \&Carp::Notify::notify;
 
-		#foreach my $var (@_){
 		while (defined (my $var = shift)){
 			if ($var eq ""){die ("Error...tried to import undefined value in $file, Line $line\n")};
 			
 			if ($var =~ /^$settables$/o){
-				$init{$var} = shift;
+				$def{$var} = shift;
 				next;
 			};
 			
-			push @storable_vars, $var if $var =~ /^[\$@%]/;
-			push @{$storable_vars{$calling_package}}, $var if $var =~ /^[\$@%]/;
+			push @storable_vars, $var if $var =~ /^[\$@%&]/;
+			push @{$storable_vars{$calling_package}}, $var if $var =~ /^[\$@%&]/;
 			
 			#see if we want to overload croak or export anything while we're at it.
-			#sub {shift->accessor($prop,  @_)};
+			
 			*{$package . "::croak"}   			= \&Carp::Notify::explode if $var eq "croak";
 			*{$package . "::carp"}   			= \&Carp::Notify::notify if $var eq "carp";
 			*{$package . "::make_storable"}   	= \&Carp::Notify::make_storable if $var eq "make_storable";
 			*{$package . "::make_unstorable"}   = \&Carp::Notify::make_unstorable if $var eq "make_unstorable";
+		
 		};
 		
-		print map {"--> $_: @{$storable_vars{$_}}\n"} keys %storable_vars;
 	};
 
 	sub store_vars {
@@ -112,14 +111,12 @@ BEGIN {
 		#foreach my $storable_var (@storable_vars){
 		foreach my $storable_var (@{$storable_vars{$calling_package}}){
 						
-			my $type = $1 if $storable_var =~ s/([\$@%])//;
+			my $type = $1 if $storable_var =~ s/([\$@%&])//;
 			
 			my $package = $calling_package . "::";
 			   $package = $1 if $storable_var =~ s/(.+::)//; 
 
 			
-			print "STORABLE : $package$storable_var\n\n";
-			
 			if ($type eq '$') {
 				my $storable_val = ${$package . "$storable_var"};
 				$stored_vars .= "\t\$${package}$storable_var : $storable_val\n";next;
@@ -135,6 +132,10 @@ BEGIN {
 				my @storable_val =  map {"\n\t\t$_ => $temp_hash{$_}"} keys %temp_hash;
 				$stored_vars .= "\t\%${package}$storable_var : @storable_val\n";next;
 			
+			}
+			elsif ($type eq '&'){
+				my $storable_val = &{$package . "$storable_var"};
+				$stored_vars .= "\t\&${package}$storable_var : $storable_val\n";next;
 			};
 
 		};
@@ -145,7 +146,7 @@ BEGIN {
 	
 	sub make_storable {
 		foreach my $var (@_){
-			push @storable_vars, $var if $var =~ /^[\$@%]/;;
+			push @storable_vars, $var if $var =~ /^[\$@%&]/;;
 		};
 		return 1;	
 	};
@@ -156,9 +157,11 @@ BEGIN {
 		return 1;
 	};
 	
-	#hee hee!  Remember, a notification is just an explosion that isn't fatal.  So we use are nifty handy dandy
+	#hee hee!  Remember, a notification is just an explosion that isn't fatal.  So we use our nifty handy dandy
 	#fatal class variable to tell explode that it's not a fatal error.  explode() will set fatal back to 1 once
 	#it realizes that errors are non-fatal.  That way a future explosion will still be fatal.
+	#
+	#and then goto &explode makes perl think it just started at the explode function.  Even caller can't catch it!
 	sub notify {
 		$Carp::Notify::fatal = 0;
 		goto &explode;
@@ -166,63 +169,90 @@ BEGIN {
 
 	sub explode {
 
-		#my %init = @_;
 		my $errors = undef;
 		
+		my %init = ();
+		
 		while (defined (my $arg = shift)) {
 			if ($arg =~ /^$settables$/o){
 				$init{$arg} = shift;
 			}
 			else {$errors .= "\t$arg\n"};
 		};
-		
-		my $log_it   	= defined $init{"log_it"} 		? $init{"log_it"}		: $def_log_it;
-		my $email_it   	= defined $init{"email_it"} 	? $init{"email_it"}		: $def_email_it;
 
-		my $store_vars	= defined $init{"store_vars"} 	? $init{"store_vars"} 	: $def_store_vars;
-		my $stack_trace = defined $init{"stack_trace"} 	? $init{"stack_trace"}	: $def_stack_trace;
-		my $store_env	= defined $init{"store_env"} 	? $init{"store_env"} 	: $def_store_env;
+		%init = (%def, %init);
 
-		my $email 	 	= $init{"email"} 			|| $def_email;
-		my $log_file	= $init{"log_file"} 		|| $def_log_file;
-		my $smtp	 	= $init{'smtp'}				|| $def_smtp;
-		
-		my $stored_vars = store_vars()  if $store_vars;
-		my $stack 		= stack_trace() if $stack_trace;
-		my $environment	= store_env()   if $store_env;
-		
-		my $die_to_stdout = defined $init{"die_to_stdout"} 	? $init{"die_to_stdout"} 	: $def_die_to_stdout;
-		my $die_quietly   = defined $init{"die_quietly"} 	? $init{"die_quietly"} 	: $def_die_quietly;
-		my $death_message = $init{'death_message'}			|| $def_death_message;
+		my $stored_vars = store_vars()  if $init{'store_vars'};
+		my $stack 		= stack_trace() if $init{'stack_trace'};
+		my $environment	= store_env()   if $init{'store_env'};
 		
 		my $message = "";
 		
-		$message .= "An error occurred on " . today() . "\n";
+		my $method = $Carp::Notify::fatal ? 'explosion' : 'notification';
+		
+		$message .= "An error via $method occurred on " . today() . "\n";
 		
 		$message .= "\n>>>>>>>>>\nERROR MESSAGES\n>>>>>>>>>\n\n$errors\n<<<<<<<<<\nEND ERROR MESSAGES\n<<<<<<<<<\n" 		if $errors;
 		$message .= "\n>>>>>>>>>\nSTORED VARIABLES\n>>>>>>>>>\n\n$stored_vars\n<<<<<<<<<\nEND STORED VARIABLES\n<<<<<<<<<\n"if $stored_vars;	
-		$message .= "\n>>>>>>>>>\nCALL STACK TRACE\n>>>>>>>>>\n\n$stack\n<<<<<<<<<\nEND CALL STACK TRACE\n<<<<<<<<<\n" 		if $stack_trace;
-		$message .= "\n>>>>>>>>>\nENVIRONMENT\n>>>>>>>>>\n\n$environment\n<<<<<<<<<\nEND ENVIRONMENT\n<<<<<<<<<\n" 			if $store_env;
+		$message .= "\n>>>>>>>>>\nCALL STACK TRACE\n>>>>>>>>>\n\n$stack\n<<<<<<<<<\nEND CALL STACK TRACE\n<<<<<<<<<\n" 		if $init{'stack_trace'};
+		$message .= "\n>>>>>>>>>\nENVIRONMENT\n>>>>>>>>>\n\n$environment\n<<<<<<<<<\nEND ENVIRONMENT\n<<<<<<<<<\n" 			if $init{'store_env'};
 		
 		log_it(
-			"log_file" => $log_file,
-			"message" => $message
-		) if $log_it;
+			"log_it"		=> $init{'log_it'},
+			"log_file"		=> $init{'log_file'},
+			
+			"log_explode"	=> $Carp::Notify::fatal   && $init{"log_explode"}	#fatal and logging explosions?
+									? $init{"log_explode"}
+									: 0,
+			"explode_log"	=> $init{'explode_log'},
+
+			"log_notify"	=> ! $Carp::Notify::fatal && $init{"log_notify"}	#non-fatal and logging notifys?
+									? $init{"log_notify"}
+									: 0,
+			"notify_log"	=> $init{"notify_log"},
+			
+			"message" 		=> $message,
+			"error_function" => $init{'error_function'}
+		);
 		
 		simple_smtp_mailer(
-			"email" => $email,
-			"message" => $message
-		) if $email_it;
+			"email" => $init{'email'},
+			"return" => $init{'return'},
+			"message" => $message,
+			"subject" => $init{'subject'},
+			"smtp" => $init{'smtp'},
+			"port" => $init{'port'},
+			"error_function" => $init{'error_function'}
+		) if $init{'email_it'};
+		
 		if ($Carp::Notify::fatal){
-			if ($die_quietly){
+			if ($init{'die_quietly'}){
 				exit;
 			}
+			elsif ($init{'death_function'}){
+				if (ref $init{'death_function'} eq 'CODE'){
+					$init{'death_function'}->(%init, 'errors' => $errors);
+				}
+				else {
+					no strict 'vars';
+					my ($calling_package) = (caller)[0];
+					my $package = $calling_package . "::";
+			   		$package = $1 if $init{'death_function'} =~ s/(.+::)//; 
+					$init{'death_function'} =~ s/^&//;
+					&{$package . $init{'death_function'}}(%init, 'errors' => $errors);
+					exit;
+				};
+			}
 			else {
-				if ($die_to_stdout){
-					print $death_message;
+				if ($init{'die_to_stdout'}){
+					print STDERR $init{'death_message'} if $init{'die_everywhere'};
+					print $init{'death_message'};
 					exit;
 				}
-				else {die $death_message};
+				else {
+					print $init{'death_message'} if $init{'die_everywhere'};
+					die $init{'death_message'};
+				};
 			};
 		}
 		else {
@@ -235,12 +265,10 @@ BEGIN {
 };
 
 
-
-
 #psst!  If you're looking for store_vars, it's up at the top wrapped up with import!	
 
 sub store_env {
-	my $env = undef;
+	my $env = '';
 	foreach (sort keys %ENV){
 		$env .= "\t$_ : $ENV{$_}\n";
 	};
@@ -254,6 +282,8 @@ sub stack_trace {
 	my @verbose_caller = ("Package: ", "Filename: ", "Line number: ", "Subroutine: ", "Has Args? : ",
 							"Want array? : ", "Evaltext: ", "Is require? : ");
 	
+	push @verbose_caller, ("Hints:  ", "Bitmask:  ") if $] >= 5.006;	#5.6 has a more verbose caller stack.
+	
 	while (my @caller = caller($caller_count++)){
 		$caller_stack .= "\t---------\n";
 		foreach (0..$#caller){
@@ -267,31 +297,35 @@ sub stack_trace {
 
 sub log_it {
 	my %init = @_;
-	
-	my $log_file = $init{log_file};
+
 	my $message  = $init{message};
 	
 	local *LOG;
+	my %pairs = (
+		"log_notify"	=> "notify_log",
+		"log_explode"	=> "explode_log",
+		"log_it"		=> "log_file"
+	);
 	
-	if (! ref $log_file){
-		open (LOG, ">>$log_file") or error ("Cannot open log file: $!");
-	}
-	else {
-		*LOG = $log_file;
-	};
-	
-	print LOG "\n__________________\n$message\n__________________\n";
-	if (! ref $log_file){close LOG or error ("Cannot close log file: $!")};
+	foreach my $permission (grep {$init{$_}} keys %pairs) {	
+		my $file = $init{$pairs{$permission}};
+		if (ref $file){
+			print $file "\n__________________\n$message\n__________________\n";
+		}
+		else {
+			open (LOG, ">>$file") or error($init{'error_function'},"Cannot open log file: $!");
+			print LOG "\n__________________\n$message\n__________________\n";
+			close LOG or error($init{'error_function'},"Cannot close log file: $!");
+		};
+		
+	 };
 };
 
 sub simple_smtp_mailer {
 
-	error ("Cannot email: Socket.pm could not load!") unless $Carp::Notify::can_email;
-
 	my %init = @_;
-	my $email	= $init{"email"} || $def_email;
-	my $smtp 	= $init{"smtp"}  || $def_smtp;
 	my $message = $init{"message"};
+	error($init{'error_function'},"Cannot email: Socket.pm could not load!") unless $Carp::Notify::can_email;
 
 	local *MAIL;
 	my $response = undef;	
@@ -301,12 +335,12 @@ sub simple_smtp_mailer {
 
 	#connect to the server
 	1 while ($s_tries-- && ! socket(MAIL, PF_INET, SOCK_STREAM, getprotobyname('tcp')));
-	return error("Socket error $!") if $s_tries < 0;
+	return error($init{'error_function'}, "Socket error $!") if $s_tries < 0;
 	
-	my $remote_address = inet_aton($smtp);
-	my $paddr = sockaddr_in(25, $remote_address);
+	my $remote_address = inet_aton($init{'smtp'});
+	my $paddr = sockaddr_in($init{'port'}, $remote_address);
 	1 while ! connect(MAIL, $paddr) && $c_tries--;
-	return error("Connect error $!") if $c_tries < 0;
+	return error($init{'error_function'}, "Connect error $!") if $c_tries < 0;
 	
 	#keep our bulk pipes piping hot.
 	select((select(MAIL), $| = 1)[0]);
@@ -316,10 +350,10 @@ sub simple_smtp_mailer {
 	my @conversation =
 		(
 			["", "No response from server: ?"],
-			["HELO $def_domain", "Mean ole' server won't say HELO: ?"],
+			["HELO $def{'domain'}", "Mean ole' server won't say HELO: ?"],
 			["RSET", "Cannot reset connection: ?"],
-			["MAIL FROM:<$def_return>", "Invalid Sender: ?"],
-			["RCPT TO:<$email>", "Invalid Recipient: ?"],
+			["MAIL FROM:<$def{'return'}>", "Invalid Sender: ?"],
+			["RCPT TO:<$init{'email'}>", "Invalid Recipient: ?"],
 			["DATA", "Not ready to accept data: ?"]
 		);
 
@@ -330,24 +364,23 @@ sub simple_smtp_mailer {
 
 		if (! $response || $response =~ /^[45]/){
 			$i_die =~ s/\?/$response/;
-			return error($i_die);
+			return error($init{'error_function'}, $i_die);
 		};
-		return error("Server disconnected: $response") if $response =~ /^221/;
+		return error($init{'error_function'}, "Server disconnected: $response") if $response =~ /^221/;
 
 	};
 	#built
 	
 	#send the data
 	print MAIL "Date: ", today();
-	print MAIL "From: $def_return";
-	print MAIL "Subject: $def_subject";
-	print MAIL "To: $email";
+	print MAIL "From: $init{'return'}";
+	print MAIL "Subject: $init{'subject'}";
+	print MAIL "To: $init{'email'}";
 	print MAIL "X-Priority:2  (High)";
-	print MAIL "X-Carp::Notify: $Carp::Notify::VERSION";
+	print MAIL "X-Carp-Notify: $Carp::Notify::VERSION";
 	
 	print MAIL "";	$message =~ s/^\./../gm;
-	$message =~ s/\r?\n/\015\012/g;
-	#print "MESSAGE\n(\n$message\n)\n";
+	$message =~ s/(\r?\n|\r)/\015\012/g;
 	
 	print MAIL $message;
 
@@ -358,29 +391,41 @@ sub simple_smtp_mailer {
 };
 
 sub today {
-	
+
 	my @months 	= qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
 	my @days 	= qw(Sun Mon Tue Wed Thu Fri Sat);
-	
-	my ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime(time);
-	
-	$hour = "0" . $hour if $hour < 10;
-	$min  = "0" . $min  if $min < 10;
-	$sec  = "0" . $sec if $sec < 10;
-	$year += 1900;		#RFC 1123 dates are 4 digit!
-	
-	my($gmin, $ghour, $gsdst) = (gmtime(time))[1,2, -1];
 
-	(my $diffhour = sprintf("%03d", $hour - $ghour)) =~ s/^0/\+/;
+	my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime(time);
+	$year += 1900;
+	my ($gmin, $ghour, $gsdst) = (gmtime(time))[1,2, -1];
 	
-	return "$days[$wday], $mday $months[$mon] $year $hour:$min:$sec " . $diffhour . sprintf("%02d", $min - $gmin);
+	my $diffhour = $hour - $ghour;
+	$diffhour = 12 - $diffhour if $diffhour > 12;
+	$diffhour = 12 + $diffhour if $diffhour < -12;
+
+	($diffhour = sprintf("%03d", $diffhour)) =~ s/^0/\+/;
 	
+	return sprintf("%s, %02d %s %04d %02d:%02d:%02d %05s",
+		$days[$wday], $mday, $months[$mon], $year + 1900, $hour, $min, $sec, $diffhour . sprintf("%02d", $min - $gmin));
 };
 
+#error does nothing unless you specify the error_function, in that case it's called with the error provided.
+sub error {
+	my ($func, $error) = @_;
+	if (ref $func eq 'CODE'){
+		$func->($error);
+	}
+	elsif ($func){
+		no strict 'refs';
+		my $calling_package = (caller(2))[0];	#eek!  This may bite me in the ass.
 
-#I haven't decided what to do with error.  It's hanging around in case I think of a clever way to handle errors within
-#the module.  But, of course, how does the error reporting module report an error reporting an error?...
-sub error { undef};
+		my ($calling_package) = (caller)[0];
+		my $package = $calling_package . "::";
+		$package = $1 if $$func =~ s/(.+::)//;
+		&{$package . $func}($error);
+	}
+	else {undef};
+};
 
 
 1;
@@ -414,7 +459,7 @@ Use it in place of die or croak, or warn or carp.
 
 =head1 REQUIRES
 
-Perl 5.004, Socket (for emailing)
+Perl 5.005, Socket (for emailing)
 
 =head1 DESCRIPTION
 
@@ -425,6 +470,7 @@ If you want the program to tell you something about an error that's non fatal (d
 there, for example), then you can have it notify you of the error but not terminate the program.
 
 Defaults are set up within the module, but they can be overridden once the module is used, or as individual explosions take place.
+B<Please> set up your appropriate defaults in the module.  It'll save you headaches later.
 
 =head1 BUILT IN STUFF
 
@@ -478,7 +524,7 @@ you can switch all of your carps to notifies by just changing which module you u
 
 =over 3
 
-=item (log_it|email_it|store_vars|stack_trace|store_env|email|log_file|smtp|die_to_stdout|die_quietly|death_message)
+=item (log_it|log_file|log_explode|explode_log|log_notify|notify_log|store_vars|stack_trace|store_env|email_it|email|return|subject|smtp|domain|port|die_to_stdout|die_quietly|die_everywhere|death_message|death_function|error_function)
 
 Example:
 
@@ -495,7 +541,7 @@ in your explode() calls, but this allows a global modification.
 
 =item log_it
 
-Flag that tells Carp::Notify whether or not to log the explosions to the log file
+Flag that tells Carp::Notify whether or not to log the explosions to the main log file
 
 =back
 
@@ -514,30 +560,61 @@ Be sure to use globrefs only explicitly in your call to explode, or to wrap the
 filehandle in a begin block before using the module.  Otherwise, you'll be trying to log to
 a non-existent file handle and consequently won't log anything.  That'd be bad.
 
-=back
-
-=over 2
-
-=item email_it
-
-Flag that tells Carp::Notify whether or not to email a user to let them know something broke
+log_file stores all notifications AND explosions
 
 =back
 
 =over 2
 
-=item email
+=item log_explode
 
-Overrides the default email address.  This is whoever the error message will get emailed to
-if U<email_it> is true.
+Flag that tells Carp::Notify whether or not to log the explosions to the explosion log
 
 =back
 
 =over 2
 
-=item smtp
+=item explode_log
 
-Allows you to set a new SMTP relay for emailing
+Overrides the default explosion log file.  This file will be opened in append mode and the explosion
+message will be stored in it, if I<log_it> is true.
+
+If you'd like, you can give explode_log a reference to a glob that is an open filehandle instead
+of a scalar containing the log name.  This is most useful if you want to redirect your explosion log
+to STDERR or to a pipe to a program.
+
+Be sure to use globrefs only explicitly in your call to explode, or to wrap the definition of the
+filehandle in a begin block before using the module.  Otherwise, you'll be trying to log to
+a non-existent file handle and consequently won't log anything.  That'd be bad.
+
+explode_log stores ONLY explosions
+
+=back
+
+=over 2
+
+=item log_notify
+
+Flag that tells Carp::Notify whether or not to log the notifications to the notification log
+
+=back
+
+=over 2
+
+=item notify_log
+
+Overrides the default notification log file.  This file will be opened in append mode and the notification
+message will be stored in it, if I<log_it> is true.
+
+If you'd like, you can give notify_log a reference to a glob that is an open filehandle instead
+of a scalar containing the log name.  This is most useful if you want to redirect your notification log
+to STDERR or to a pipe to a program.
+
+Be sure to use globrefs only explicitly in your call to notify, or to wrap the definition of the
+filehandle in a begin block before using the module.  Otherwise, you'll be trying to log to
+a non-existent file handle and consequently won't log anything.  That'd be bad.
+
+notify_log stores ONLY notifications
 
 =back
 
@@ -567,6 +644,65 @@ Flag that tells Carp::Notify whether or not to store the environment variables i
 
 =over 2
 
+=item email_it
+
+Flag that tells Carp::Notify whether or not to email a user to let them know something broke
+
+=back
+
+=over 2
+
+=item email
+
+Overrides the default email address.  This is whoever the error message will get emailed to
+if U<email_it> is true.
+
+=back
+
+=over 2
+
+=item return
+
+Overrides the default return address.  This is whoever the error message will be coming from
+if U<email_it> is true.
+
+=back
+
+=over 2
+
+=item subject
+
+Overrides the default subject.  This is the subject of the email message
+if U<email_it> is true.
+
+=back
+
+=over 2
+
+=item smtp
+
+Allows you to set a new SMTP relay for emailing
+
+=back
+
+=over 2
+
+=item port
+
+Allows you to set a new SMTP port for emailing
+
+=back
+
+=over 2
+
+=item domain
+
+Allows you to set a new SMTP domain for emailing
+
+=back
+
+=over 2
+
 =item die_to_stdout
 
 Allows you to terminate your program by displaying an error to STDOUT, not to STDERR.
@@ -578,6 +714,15 @@ Allows you to terminate your program by displaying an error to STDOUT, not to ST
 =item die_quietly
 
 Terminates your program without displaying any message to STDOUT or STDERR.
+
+=back
+
+=over 2
+
+=item die_everywhere
+
+Terminates your program and displays error message to STDOUT and STDERR.
+
 =back
 
 =over 2
@@ -585,6 +730,33 @@ Terminates your program without displaying any message to STDOUT or STDERR.
 =item death_message
 
 The message that is printed to the appropriate location, unless we're dying quietly.
+
+=back
+
+=over 2
+
+=item death_function
+
+death_function can be called instead of death_message, for a more dynamic error message.
+It is handed a hash all of the Carp::Notify variables along with an additional key 'errors' containing
+the errors in the current explosion or notification.
+
+death_function's use will cause the program to terminate with a simple 'exit'.  You'll have to die within death_function
+yourself if you want to pass on that way.
+
+death_function may be a coderef or a string containing a function name ('death_function', 'main::death', '&death', etc.)
+
+=back
+
+=over 2
+
+=item error_function
+
+error_function is called by Carp::Notify if it encounters an error (cannot open log file, or cannot connect
+to SMTP server, for instance), it is given one parameter, the literal error that occurred.  You may set this parameter
+to do additional processing if Carp::Notify cannot complete error notification.
+
+error_function may be a coderef or a string containing a function name ('error_function', 'main::error', '&error', etc.)
 =back
 
 =back
@@ -606,15 +778,14 @@ typed something in single quotes, for instance.  For example,
  
 will write out the values "$scalar : some_value" and "@array : val1 val2 val3" to the log file.
 
-This can also only be used to store global variables. Dynamic or lexical variables need to be explicitly placed in explode() calls.
+This can B<only> be used to store global variables. Dynamic or lexical variables need to be explicitly placed in explode() calls.
 
 You can store variables from other packages if you'd like:
 
 use Carp::Notify ('$other_package::scalar', '@some::nested::package::array');
 
-Only I<global> scalars, arrays, and hashes may be stored.
-
-=back
+Only I<global> scalars, arrays, and hashes may be stored in 1.00. 1.10 allows you to store a function call as well, the function will
+be called with the same arguements that death_message would receive.  Its return value will be stored along with other stored variables.
 
 =back
 
@@ -700,7 +871,7 @@ obvious for anything (cron jobs, command line utilities, as well as CGIs).
 Carp::Notify also can store more information with less interaction from the programmer.  Plus it will email you, if you'd like
 to let you know that something bad happened.
 
-As I understand it, CGI::Carp is a subset feature-wise of Carp::Notify.  If CGI::Carp is working fine for you, great continue to use
+As I understand it, CGI::Carp is a subset feature-wise of Carp::Notify.  If CGI::Carp is working fine for you -- great, continue to use
 it.  If you want more flexible error notification, then try out Carp::Notify.
 
 B<But I can send email with CGI::Carp by opening up a pipe to send mail and using that as my error log.  What do you have
@@ -712,8 +883,8 @@ wouldn't have thought of it.  Besides, it's still more of a hassle than just usi
 B<Why are your stored variables kept in an array instead of a hash?  Hashes are quicker to delete from, after all>
 
 While it is definitely true that variables can be unstored a little quicker in a hash, I figured that stored variables 
-will only rarely be unstored later.  Arrays are quicker for storing and accessing the items later.   I'll live with the
-slight performance hit for the rarer case.
+will only rarely be unstored later.  Arrays are quicker for storing and accessing the items later, since they're only accessed en masse.
+I'll live with the slight performance hit for the rarer case.
 
 B<Can I store variables that are in another package from the one that called Carp::Notify?>
 
@@ -742,7 +913,7 @@ Mail::Bulkmail and Text::Flowchart.
 
 B<Was that a shameless plug?>
 
-Why yes, it was.
+Why yes, yes it was.
 
 =head1 Examples
 
@@ -752,26 +923,54 @@ Why yes, it was.
  #email it to a different address, and don't log it.
  use Carp::Notify ("email" => 'thomasoniii@yahoo.com', 'log_it' => 0);
 
+ #don't use global log file, log explosions and notifys separately.
+ use Carp::Notify ('log_it' => 0, 'log_explode' => 1, 'log_notify => 1);
+
  #die with an explosion.
  explode("Ye gods!  An error!");
  
+ #die with an explosion and a different email subject.
+ explode ('subject' => "HOLY SHIT THIS IS BAD!", "The reactor core breached.");
+ 
  #explode, but do it quietly.
  explode ("die_quietly" => 1, "Ye gods!  An error!");
  
  #notify someone of a problem, but keep the program running
  notify ("Ye gods!  A little error!");
 
+
 =head1 Version History
 
 =over 11
 
+v1.10 - April 13, 2001 - 
+
+* Long overdue re-write/enhancements.
+		
+* Cleaned up the internals.
+		
+* Added support for different log files for notifys/explodes.
+		
+* Added support for changing all the defaults externally.
+
+* Improved today() function with Mail::Bulkmail's date generation.
+		
+* Added death_function to call upon dying instead of death_message.
+		
+* Added error_function to help handle unforeseen problems.
+
+* Perl 5.6's 'caller' function returns an extra two fields of information.  Jeff Boes pointed this
+out and extra-helpfully provided a patch for it as well.
+                
+* Added the ability to config the SMTP port.  I was just being lazy in v1.00.  :*)
+
 v1.00 - August 10, 2000 - Changed the name from Explode to Carp::Notify.  It's more descriptive and I don't create a new namespace.
 
 v1.00 FC1 - June 9, 2000 - First publically available version.
 
 =head1 COPYRIGHT (again)
 
-Copyright (c) 2000 James A Thomason III (thomasoniii@yahoo.com). All rights reserved.
+Copyright (c) 2000, 2001 James A Thomason III (thomasoniii@yahoo.com). All rights reserved.
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
@@ -783,4 +982,3 @@ made your life easier.  :-)
 
 =cut
 
-
